/*-
 *
 * Copyright 2004-2010 Bruce Cran <bruce@cran.org.uk>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/hdreg.h>

#include "../mi/atagen.h"

/* send a command to the drive */
int
ata_cmd(ATA *ata, int cmd, int feature, int feature_val)
{
	int rc = 0;

	ata->atacmd.feature = feature;

	if(feature != 0)
		ata->atacmd.sector_number = feature;
	else
		ata->atacmd.sector_count = 1;

	ata->atacmd.cmd = cmd;
	rc = ioctl( ata->ata_fd, HDIO_DRIVE_CMD, &ata->atacmd );

	return rc;
}

/* initialize the ata_cmd structure with supplied values */
void
ata_init(ATA *ata)
{
	memset(&ata->atacmd, 0, sizeof(struct ata_cmd));
}

char*
ata_setup_buffer(ATA *ata, int nbytes)
{
	return (char*) ata->atacmd.buf;
}

ATA*
ata_open(const char *path)
{
	ATA *ata = NULL;

	ata = malloc(sizeof(ATA));
	if (ata == NULL)
		return NULL;

	ata->disk_protocol = DISK_PROTO_ATA;
	ata->ata_fd = open(path, O_RDONLY);
	if (ata->ata_fd < 0) {
		free(ata);
		return NULL;
	}

	return ata;
}

void
ata_close(ATA *ata)
{
	if (ata->ata_fd >= 0)
		close(ata->ata_fd);

	free(ata);
}

void
ata_free(ATA *ata)
{
  /* nothing to do */
}
